@@ -23,13 +23,13 @@
 
 /**
  * SECTION:element-pulsesink
+ * @title: pulsesink
  * @see_also: pulsesrc
  *
  * This element outputs audio to a
- * <ulink href="http://www.pulseaudio.org">PulseAudio sound server</ulink>.
+ * [PulseAudio sound server](http://www.pulseaudio.org).
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! pulsesink
  * ]| Play an Ogg/Vorbis file.
@@ -40,7 +40,7 @@
  * gst-launch-1.0 -v audiotestsrc ! pulsesink stream-properties="props,media.title=test"
  * ]| Play a sine wave and set a stream property. The property can be checked
  * with "pactl list".
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,12 +53,13 @@
 #include <gst/base/gstbasesink.h>
 #include <gst/gsttaglist.h>
 #include <gst/audio/audio.h>
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 
 #include <gst/pbutils/pbutils.h>        /* only used for GST_PLUGINS_BASE_VERSION_* */
 
 #include <gst/glib-compat-private.h>
 
+#include "gstpulseelements.h"
 #include "pulsesink.h"
 #include "pulseutil.h"
 
@@ -154,7 +155,7 @@ static guint mainloop_ref_ct = 0;
 static GMutex pa_shared_resource_mutex;
 
 /* We keep a custom ringbuffer that is backed up by data allocated by
- * pulseaudio. We must also overide the commit function to write into
+ * pulseaudio. We must also override the commit function to write into
  * pulseaudio memory instead. */
 struct _GstPulseRingBuffer
 {
@@ -177,9 +178,9 @@ struct _GstPulseRingBuffer
   gint64 m_offset;
   gint64 m_lastoffset;
 
-  gboolean corked:1;
-  gboolean in_commit:1;
-  gboolean paused:1;
+  gboolean corked;
+  gboolean in_commit;
+  gboolean paused;
 };
 struct _GstPulseRingBufferClass
 {
@@ -322,6 +323,7 @@ gst_pulsering_destroy_stream (GstPulseRingBuffer * pbuf)
     pa_stream_set_write_callback (pbuf->stream, NULL, NULL);
     pa_stream_set_underflow_callback (pbuf->stream, NULL, NULL);
     pa_stream_set_overflow_callback (pbuf->stream, NULL, NULL);
+    pa_stream_set_latency_update_callback (pbuf->stream, NULL, NULL);
 
     pa_stream_unref (pbuf->stream);
     pbuf->stream = NULL;
@@ -369,7 +371,7 @@ gst_pulsering_destroy_context (GstPulseRingBuffer * pbuf)
         g_hash_table_remove (gst_pulse_shared_contexts, pbuf->context_name);
 
         pa_context_unref (pctx->context);
-        g_slice_free (GstPulseContext, pctx);
+        g_free (pctx);
       }
     }
     g_free (pbuf->context_name);
@@ -526,7 +528,7 @@ gst_pulseringbuffer_open_device (GstAudioRingBuffer * buf)
 
   pctx = g_hash_table_lookup (gst_pulse_shared_contexts, pbuf->context_name);
   if (pctx == NULL) {
-    pctx = g_slice_new0 (GstPulseContext);
+    pctx = g_new0 (GstPulseContext, 1);
 
     /* get the mainloop api and create a context */
     GST_INFO_OBJECT (psink, "new context with name %s, pbuf=%p, pctx=%p",
@@ -545,7 +547,7 @@ gst_pulseringbuffer_open_device (GstAudioRingBuffer * buf)
         gst_pulsering_context_subscribe_cb, pctx);
 
     /* try to connect to the server and wait for completion, we don't want to
-     * autospawn a deamon */
+     * autospawn a daemon */
     GST_LOG_OBJECT (psink, "connect to server %s",
         GST_STR_NULL (psink->server));
     if (pa_context_connect (pctx->context, psink->server,
@@ -606,7 +608,7 @@ create_failed:
   {
     GST_ELEMENT_ERROR (psink, RESOURCE, FAILED,
         ("Failed to create context"), (NULL));
-    g_slice_free (GstPulseContext, pctx);
+    g_free (pctx);
     goto unlock_and_fail;
   }
 connect_failed:
@@ -685,7 +687,7 @@ gst_pulsering_stream_request_cb (pa_stream * s, size_t length, void *userdata)
 
   if (pbuf->in_commit && (length >= rbuf->spec.segsize)) {
     /* only signal when we are waiting in the commit thread
-     * and got request for atleast a segment */
+     * and got request for at least a segment */
     pa_threaded_mainloop_signal (mainloop, 0);
   }
 }
@@ -742,8 +744,8 @@ gst_pulsering_stream_latency_cb (pa_stream * s, void *userdata)
      * less. One concern here is that latency updates happen every 100ms, which
      * means segdone is not updated very often, but increasing the update
      * frequency would mean more communication overhead. */
-    g_atomic_int_set (&ringbuf->segdone,
-        (int) gst_util_uint64_scale_ceil (info->read_index, 1,
+    gst_audio_ring_buffer_set_segdone (ringbuf,
+        gst_util_uint64_scale_ceil (info->read_index, 1,
             ringbuf->spec.segsize));
   }
 
@@ -1185,10 +1187,9 @@ gst_pulseringbuffer_clear (GstAudioRingBuffer * buf)
   pa_threaded_mainloop_unlock (mainloop);
 }
 
-#if 0
-/* called from pulse thread with the mainloop lock */
+/* called from pulse thread WITHOUT the mainloop lock */
 static void
-mainloop_enter_defer_cb (pa_mainloop_api * api, void *userdata)
+mainloop_enter_defer_cb (pa_threaded_mainloop * loop, void *userdata)
 {
   GstPulseSink *pulsesink = GST_PULSESINK (userdata);
   GstMessage *message;
@@ -1204,11 +1205,10 @@ mainloop_enter_defer_cb (pa_mainloop_api * api, void *userdata)
 
   gst_element_post_message (GST_ELEMENT (pulsesink), message);
 
-  g_return_if_fail (pulsesink->defer_pending);
-  pulsesink->defer_pending--;
+  g_return_if_fail (g_atomic_int_get (&pulsesink->defer_pending));
+  g_atomic_int_dec_and_test (&pulsesink->defer_pending);
   pa_threaded_mainloop_signal (mainloop, 0);
 }
-#endif
 
 /* start/resume playback ASAP, we don't uncork here but in the commit method */
 static gboolean
@@ -1230,19 +1230,14 @@ gst_pulseringbuffer_start (GstAudioRingBuffer * buf)
       g_atomic_int_get (&GST_AUDIO_BASE_SINK (psink)->eos_rendering))
     gst_pulsering_set_corked (pbuf, FALSE, FALSE);
 
-#if 0
+#if PA_CHECK_VERSION(13, 0, 0)
   GST_DEBUG_OBJECT (psink, "scheduling stream status");
-  psink->defer_pending++;
-  pa_mainloop_api_once (pa_threaded_mainloop_get_api (mainloop),
-      mainloop_enter_defer_cb, psink);
-
-  /* Wait for the stream status message to be posted. This needs to be done
-   * synchronously because the callback will take the mainloop lock
-   * (implicitly) and then take the GST_OBJECT_LOCK. Everywhere else, we take
-   * the locks in the reverse order, so not doing this synchronously could
-   * cause a deadlock. */
-  GST_DEBUG_OBJECT (psink, "waiting for stream status (ENTER) to be posted");
-  pa_threaded_mainloop_wait (mainloop);
+  g_atomic_int_inc (&psink->defer_pending);
+  /* We'll emit this in the mainloop thread so applications have a callback in
+   * that context. We emit this without the mainloop locked -- if we did not,
+   * then the mainloop lock would be implicitly taken before the object lock
+   * (during message emission), causing a deadlock */
+  pa_threaded_mainloop_once_unlocked (mainloop, mainloop_enter_defer_cb, psink);
 #endif
 
   pa_threaded_mainloop_unlock (mainloop);
@@ -1276,10 +1271,9 @@ gst_pulseringbuffer_pause (GstAudioRingBuffer * buf)
   return res;
 }
 
-#if 0
-/* called from pulse thread with the mainloop lock */
+/* called from pulse thread WITHOUT the mainloop lock */
 static void
-mainloop_leave_defer_cb (pa_mainloop_api * api, void *userdata)
+mainloop_leave_defer_cb (pa_threaded_mainloop * loop, void *userdata)
 {
   GstPulseSink *pulsesink = GST_PULSESINK (userdata);
   GstMessage *message;
@@ -1295,11 +1289,10 @@ mainloop_leave_defer_cb (pa_mainloop_api * api, void *userdata)
 
   gst_element_post_message (GST_ELEMENT (pulsesink), message);
 
-  g_return_if_fail (pulsesink->defer_pending);
-  pulsesink->defer_pending--;
+  g_return_if_fail (g_atomic_int_get (&pulsesink->defer_pending));
+  g_atomic_int_dec_and_test (&pulsesink->defer_pending);
   pa_threaded_mainloop_signal (mainloop, 0);
 }
-#endif
 
 /* stop playback, we flush everything. */
 static gboolean
@@ -1347,19 +1340,14 @@ cleanup:
     pa_operation_cancel (o);
     pa_operation_unref (o);
   }
-#if 0
+#if PA_CHECK_VERSION(13, 0, 0)
   GST_DEBUG_OBJECT (psink, "scheduling stream status");
-  psink->defer_pending++;
-  pa_mainloop_api_once (pa_threaded_mainloop_get_api (mainloop),
-      mainloop_leave_defer_cb, psink);
-
-  /* Wait for the stream status message to be posted. This needs to be done
-   * synchronously because the callback will take the mainloop lock
-   * (implicitly) and then take the GST_OBJECT_LOCK. Everywhere else, we take
-   * the locks in the reverse order, so not doing this synchronously could
-   * cause a deadlock. */
-  GST_DEBUG_OBJECT (psink, "waiting for stream status (LEAVE) to be posted");
-  pa_threaded_mainloop_wait (mainloop);
+  g_atomic_int_inc (&psink->defer_pending);
+  /* We'll emit this in the mainloop thread so applications have a callback in
+   * that context. We emit this without the mainloop locked -- if we did not,
+   * then the mainloop lock would be implicitly taken before the object lock
+   * (during message emission), causing a deadlock */
+  pa_threaded_mainloop_once_unlocked (mainloop, mainloop_leave_defer_cb, psink);
 #endif
 
   pa_threaded_mainloop_unlock (mainloop);
@@ -1565,7 +1553,7 @@ gst_pulseringbuffer_commit (GstAudioRingBuffer * buf, guint64 * sample,
           goto fake_done;
         }
 
-        if (pbuf->m_writable == (size_t) - 1)
+        if (pbuf->m_writable == (size_t) -1)
           goto writable_size_failed;
 
         pbuf->m_writable /= bpf;
@@ -1820,6 +1808,8 @@ G_DEFINE_TYPE_WITH_CODE (GstPulseSink, gst_pulsesink, GST_TYPE_AUDIO_BASE_SINK,
     gst_pulsesink_init_contexts ();
     G_IMPLEMENT_INTERFACE (GST_TYPE_STREAM_VOLUME, NULL)
     );
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (pulsesink, "pulsesink",
+    GST_RANK_PRIMARY + 10, GST_TYPE_PULSESINK, pulse_element_init (plugin));
 
 static GstAudioRingBuffer *
 gst_pulsesink_create_ringbuffer (GstAudioBaseSink * sink)
@@ -1959,7 +1949,7 @@ gst_pulsesink_class_init (GstPulseSinkClass * klass)
    * GstPulseSink:stream-properties:
    *
    * List of pulseaudio stream properties. A list of defined properties can be
-   * found in the <ulink url="http://0pointer.de/lennart/projects/pulseaudio/doxygen/proplist_8h.html">pulseaudio api docs</ulink>.
+   * found in the [pulseaudio api docs](http://0pointer.de/lennart/projects/pulseaudio/doxygen/proplist_8h.html).
    *
    * Below is an example for registering as a music application to pulseaudio.
    * |[
@@ -2431,7 +2421,7 @@ gst_pulsesink_set_volume (GstPulseSink * psink, gdouble volume)
   if (pbuf->is_pcm)
     gst_pulse_cvolume_from_linear (&v, pbuf->channels, volume);
   else
-    /* FIXME: this will eventually be superceded by checks to see if the volume
+    /* FIXME: this will eventually be superseded by checks to see if the volume
      * is readable/writable */
     goto unlock;
 
@@ -3003,7 +2993,7 @@ gst_pulsesink_change_props (GstPulseSink * psink, GstTagList * l)
   static const gchar *const map[] = {
     GST_TAG_TITLE, PA_PROP_MEDIA_TITLE,
 
-    /* might get overriden in the next iteration by GST_TAG_ARTIST */
+    /* might get overridden in the next iteration by GST_TAG_ARTIST */
     GST_TAG_PERFORMER, PA_PROP_MEDIA_ARTIST,
 
     GST_TAG_ARTIST, PA_PROP_MEDIA_ARTIST,
@@ -3210,7 +3200,7 @@ gst_pulsesink_release_mainloop (GstPulseSink * psink)
     return;
 
   pa_threaded_mainloop_lock (mainloop);
-  while (psink->defer_pending) {
+  while (g_atomic_int_get (&psink->defer_pending)) {
     GST_DEBUG_OBJECT (psink, "waiting for stream status message emission");
     pa_threaded_mainloop_wait (mainloop);
   }
