@@ -56,6 +56,9 @@ GST_DEBUG_CATEGORY_EXTERN (equalizer_debug);
 #define gst_iir_equalizer_3bands_parent_class parent_class
 G_DEFINE_TYPE (GstIirEqualizer3Bands, gst_iir_equalizer_3bands,
     GST_TYPE_IIR_EQUALIZER);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (equalizer_3bands, "equalizer-3bands",
+    GST_RANK_NONE, GST_TYPE_IIR_EQUALIZER_3BANDS,
+    equalizer_element_init (plugin));
 
 /* equalizer implementation */
 
