@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_SHAPE_WIPE_H__
@@ -76,6 +76,8 @@ struct _GstShapeWipeClass
 
 GType gst_shape_wipe_get_type (void);
 
+GST_ELEMENT_REGISTER_DECLARE (shapewipe);
+
 G_END_DECLS
 
 #endif /* __GST_SHAPE_WIPE_H__ */
