@@ -91,6 +91,8 @@ struct _GstCoreAudio
   gboolean is_src;
   gboolean is_passthrough;
   AudioDeviceID device_id;
+  char *unique_id;
+  gboolean is_default;
   gboolean cached_caps_valid; /* thread-safe flag */
   GstCaps *cached_caps;
   gint stream_idx;
@@ -120,6 +122,7 @@ struct _GstCoreAudio
 
 #ifdef HAVE_IOS
   gdouble first_sample_time;
+  gboolean configure_session;
 #endif
 };
 
@@ -132,8 +135,6 @@ GType gst_core_audio_get_type                                (void);
 
 void gst_core_audio_init_debug (void);
 
-GstCoreAudio * gst_core_audio_new                            (GstObject *osxbuf);
-
 gboolean gst_core_audio_open                                 (GstCoreAudio *core_audio);
 
 gboolean gst_core_audio_close                                (GstCoreAudio *core_audio);
