@@ -29,9 +29,6 @@
 #include <gstv4l2object.h>
 #include <gstv4l2bufferpool.h>
 
-GST_DEBUG_CATEGORY_EXTERN (v4l2sink_debug);
-
-
 G_BEGIN_DECLS
 
 #define GST_TYPE_V4L2SINK \
@@ -54,7 +51,6 @@ struct _GstV4l2Sink {
 
   /*< private >*/
   GstV4l2Object * v4l2object;
-  GstCaps *probed_caps;         /* all supported caps of underlying v4l2 device */
 
   gint video_width, video_height;      /* original (unscaled) video w/h */
 
