@@ -29,8 +29,6 @@
 # include <valgrind/valgrind.h>
 #endif
 
-#include <unistd.h>
-
 #include <gst/check/gstcheck.h>
 #include <gst/check/gstconsistencychecker.h>
 #include <gst/base/gstbasesrc.h>
@@ -591,7 +589,7 @@ GST_START_TEST (test_remove_pad)
   fail_unless (res == TRUE, NULL);
 
   /* create an unconnected sinkpad in videomixer */
-  pad = gst_element_get_request_pad (videomixer, "sink_%u");
+  pad = gst_element_request_pad_simple (videomixer, "sink_%u");
   fail_if (pad == NULL, NULL);
 
   srcpad = gst_element_get_static_pad (videomixer, "src");
@@ -695,7 +693,7 @@ GST_START_TEST (test_clip)
 
   /* create an unconnected sinkpad in videomixer, should also automatically activate
    * the pad */
-  sinkpad = gst_element_get_request_pad (videomixer, "sink_%u");
+  sinkpad = gst_element_request_pad_simple (videomixer, "sink_%u");
   fail_if (sinkpad == NULL, NULL);
 
   gst_pad_send_event (sinkpad, gst_event_new_stream_start ("test"));
@@ -973,6 +971,7 @@ GST_START_TEST (test_loop)
 
 GST_END_TEST;
 
+#if 0
 GST_START_TEST (test_flush_start_flush_stop)
 {
   GstPadTemplate *sink_template;
@@ -1025,7 +1024,7 @@ GST_START_TEST (test_flush_start_flush_stop)
 }
 
 GST_END_TEST;
-
+#endif
 
 static Suite *
 videomixer_suite (void)
@@ -1044,7 +1043,14 @@ videomixer_suite (void)
   tcase_add_test (tc_chain, test_duration_is_max);
   tcase_add_test (tc_chain, test_duration_unknown_overrides);
   tcase_add_test (tc_chain, test_loop);
+  /* This test is racy and occasionally fails in interesting ways
+   * just like the corresponding adder test does/did, see
+   * https://bugzilla.gnome.org/show_bug.cgi?id=708891
+   * It's unlikely that it will ever be fixed for videomixer/collectpads,
+   * as it works fine with compositor */
+#if 0
   tcase_add_test (tc_chain, test_flush_start_flush_stop);
+#endif
 
   /* Use a longer timeout */
 #ifdef HAVE_VALGRIND
